@@ -6,7 +6,7 @@ subst - Greple module for text search and substitution
 
 =head1 VERSION
 
-Version 2.27
+Version 2.28
 
 =head1 SYNOPSIS
 
@@ -320,7 +320,7 @@ it under the same terms as Perl itself.
 use v5.14;
 package App::Greple::subst;
 
-our $VERSION = '2.27';
+our $VERSION = '2.28';
 
 use warnings;
 use utf8;
@@ -559,73 +559,8 @@ sub read_dict_fh {
     }
 }
 
-sub mix_regions {
-    my $option = ref $_[0] eq 'HASH' ? shift : {};
-    my($old, $new) = @_;
-    return () if @$new == 0;
-    my @old = $option->{destructive} ? @{$old} : map [ @$_ ], @{$old};
-    my @new = $option->{destructive} ? @{$new} : map [ @$_ ], @{$new};
-    unless ($option->{nosort}) {
-	@new = sort({$a->[0] <=> $b->[0] || $b->[1] <=> $a->[1]
-			 ||  (@$a > 2 ? $a->[2] <=> $b->[2] : 0) }
-		    @new);
-    }
-    my @out;
-    my($include, $overlap) = @{$option}{qw(include overlap)};
-    while (@old and @new) {
-	while (@old and $old[0][1] <= $new[0][0]) {
-	    push @out, shift @old;
-	}
-	last if @old == 0;
-	while (@new and $new[0][1] <= $old[0][0]) {
-	    push @out, shift @new;
-	}
-	while (@new and $new[0][0] < $old[0][1]) {
-	    if ($old[0][0] <= $new[0][0] and $new[0][1] <= $old[0][1]) {
-		push @$include, [ $new[0], $old[0] ] if $include;
-	    } else {
-		push @$overlap, [ $new[0], $old[0] ] if $overlap;
-	    }
-	    shift @new;
-	}
-    }
-    @$old = ( @out, @old, @new );
-}
-
-sub sieve_regions {
-    my $option = ref $_[0] eq 'HASH' ? shift : {};
-    my($old, $new) = @_;
-    return () if @$new == 0;
-    my @old = $option->{destructive} ? @{$old} : map [ @$_ ], @{$old};
-    my @new = $option->{destructive} ? @{$new} : map [ @$_ ], @{$new};
-    unless ($option->{nosort}) {
-	@new = sort({$a->[0] <=> $b->[0] || $b->[1] <=> $a->[1]
-			 ||  (@$a > 2 ? $a->[2] <=> $b->[2] : 0) }
-		    @new);
-    }
-    my @out;
-    my($include, $overlap) = @{$option}{qw(include overlap)};
-    while (@old and @new) {
-	while (@old and $old[0][1] <= $new[0][0]) {
-	    shift @old;
-	}
-	last if @old == 0;
-	while (@new and $new[0][1] <= $old[0][0]) {
-	    push @out, shift @new;
-	}
-	while (@new and $new[0][0] < $old[0][1]) {
-	    if ($old[0][0] <= $new[0][0] and $new[0][1] <= $old[0][1]) {
-		push @$include, [ $new[0], $old[0] ] if $include;
-	    } else {
-		push @$overlap, [ $new[0], $old[0] ] if $overlap;
-	    }
-	    shift @new;
-	}
-    }
-    ( @out, @new );
-}
-
-use App::Greple::Regions qw(match_regions merge_regions);
+use App::Greple::Regions qw(match_regions merge_regions filter_regions);
+use List::MoreUtils qw(pairwise);
 
 sub subst_search {
     my $text = $_;
@@ -648,28 +583,24 @@ sub subst_search {
 	##
 	## Remove all overlapped matches.
 	##
-	@match = sieve_regions {
-	    overlap => ( my $overlap = [] ),
-	    include => ( my $include = [] ),
-	    nosort  => 1,
-	}, \@matched, \@match;
+	my($in, $over, $out, $im, $om) = filter_regions \@match, \@matched;
+	@match = @$out;
 	for my $warn (
-	    [ "Overlap", $overlap, $opt_warn_overlap ],
-	    [ "Include", $include, $opt_warn_include ],
+	    [ "Include", $in,   $im, $opt_warn_include ],
+	    [ "Overlap", $over, $om, $opt_warn_overlap ],
 	    ) {
-	    my($kind, $list, $show) = @$warn;
-	    next unless $show;
-	    for my $info (@$list) {
-		my($new, $old) = @$info;
+	    my($kind, $list, $match, $show) = @$warn;
+	    $show and @$list or next;
+	    pairwise {
 		warn sprintf("%s \"%s\" with \"%s\" by #%d /%s/ in %s at %d\n",
 			     $kind,
-			     substr($_, $new->[0], $new->[1] - $new->[0]),
-			     substr($_, $old->[0], $old->[1] - $old->[0]),
+			     substr($_, $a->[0], $a->[1] - $a->[0]),
+			     substr($_, $b->[0], $b->[1] - $b->[0]),
 			     $index + 1, $p->string,
 			     $current_file,
-			     $new->[0],
+			     $a->[0],
 		    );
-	    }
+	    } @$list, @$match;
 	}
 
 	$stat{total}++;
